--[[
	Rematch

	09/11/2013 1.0.5 toc update for 5.4
  08/06/2013 1.0.4 fix for saved pets caged or missing
  05/21/2013 1.0.3 toc update for 5.3
  03/16/2013 1.0.2 added 'Auto Show' option, changed UI a bit
	03/13/2013 1.0.1 initial release
]]

BINDING_HEADER_REMATCH = "Rematch"

local rematch = CreateFrame("Frame","Rematch",UIParent)
rematch:Hide()

rematch.events = {} -- OnEvent handlers
rematch:SetScript("OnEvent",function(self,event,...)
	if rematch.events[event] then
		rematch.events[event](self,...)
	end
end)
rematch:RegisterEvent("PLAYER_LOGIN")

rematch:SetScript("OnShow",function(self)
	self:UpdatePets() -- top panel
	self:UpdateTarget() -- bottom panel
	self:RegisterEvent("UPDATE_SUMMONPETS_ACTION")
end)
rematch:SetScript("OnHide",function(self)
	self:UnregisterEvent("UPDATE_SUMMONPETS_ACTION")
end)

function rematch.events.PLAYER_LOGIN(self)
	RematchSaved = RematchSaved or {} -- savedvariable
	RematchSaved.Settings = RematchSaved.Settings or {}
	self:BuildUI()
	self:BuildTimer()
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
end
function rematch.events.PLAYER_TARGET_CHANGED(self)
	self:UpdateTarget()
	if RematchSaved.Settings.AutoShow and not self:IsVisible() and self:PetsNeedLoading() then
		self:Show()
	end
end
function rematch.events.UPDATE_SUMMONPETS_ACTION(self,...)
	self:UpdatePets()
end

function rematch:Toggle()
	if self:IsVisible() then
		self:Hide()
	else
		self:Show()
	end
end

function rematch:StartMovingFrame()
	Rematch:StartMoving()
end
function rematch:StopMovingFrame()
	Rematch:StopMovingOrSizing()
	RematchSaved.Settings.XPos = Rematch:GetLeft()
	RematchSaved.Settings.YPos = Rematch:GetBottom()
end

function rematch:UpdatePets()
	self:DesaturatePets(self.petsLoading:IsVisible())
	self.petAbilityList = self.petAbilityList or {}
	self.petAbilityLevels = self.petAbilityLevels or {}
	for i=1,3 do
	  local a,guid = {}
	  guid,a[1],a[2],a[3] = C_PetJournal.GetPetLoadOutInfo(i)
		if guid then
			self.current[i].petID = guid
			local speciesID,_,level,_,_,_,_,_,icon = C_PetJournal.GetPetInfoByPetID(guid)
		  self.current[i].icon:SetTexture(icon)
			self.current[i].icon:Show()
		  for j=1,3 do
				self.current[i].ability[j].abilityID = nil
				self.current[i].ability[j].icon:Hide()
				C_PetJournal.GetPetAbilityList(speciesID,self.petAbilityList,self.petAbilityLevels)
				for k=1,#self.petAbilityList do
					if a[j]==self.petAbilityList[k] and level>=self.petAbilityLevels[k] then
						self.current[i].ability[j].abilityID = a[j]
				    self.current[i].ability[j].icon:SetTexture((select(2,C_PetJournal.GetPetAbilityInfo(a[j]))))
						self.current[i].ability[j].icon:Show()
						break
					end
				end
			end
	  else
			self.current[i].petID = nil
			self.current[i].icon:Hide()
		end
		self:SetPetHealth(i)
	end
	if RematchSaved.Settings.KeepFirst then
		self.lock:Show()
		self.keepButton:SetChecked(1)
	else
		self.lock:Hide()
		self.keepButton:SetChecked(0)
	end
end

function rematch:UpdateTarget()
	self.target = UnitName("target") or "<no target>"
	if self:IsVisible() then
		self.savedheadertext:SetText("Saved for "..self.target)
		local saved = RematchSaved[self.target]
		if saved then
			for i=1,3 do
				local icon = select(9,C_PetJournal.GetPetInfoByPetID(saved[i][1]))
				self.saved[i].petID = saved[i][1]
				self.saved[i].icon:SetTexture(icon)
				self.saved[i].icon:Show()
			end
			self.loadButton:Enable()
			self.unsaveButton:Enable()
		else
			for i=1,3 do
				self.saved[i].petID = nil
				self.saved[i].icon:Hide()
			end
			self.loadButton:Disable()
			self.unsaveButton:Disable()
		end
	end
	self.autoButton:SetChecked(RematchSaved.Settings.AutoShow)
end

function rematch:SetPetHealth(pet,override)
	local percent
	local health = self.current[pet].health
	health:Hide()
	self.current[pet].dead:Hide()
	if self.current[pet].petID then
		local hp,hpmax = C_PetJournal.GetPetStats(self.current[pet].petID)
		local desaturate
		if hp>0 then
			local percent = hp/hpmax
			if override then
				percent = override
			end
			health:SetWidth(38*percent)
			health:Show()
		else
			self.current[pet].dead:Show()
			desaturate = 1
		end
		-- desaturate pet if it's dead, saturate if alive
		self.current[pet].icon:SetDesaturated(desaturate)
		for i=1,3 do
			self.current[pet].ability[i].icon:SetDesaturated(desaturate)
		end
	end
end

function rematch:DesaturatePets(desaturate)
	self.topbg:SetDesaturated(desaturate)
	local rb = desaturate and .5 or 0
	local g = desaturate and .5 or 1
	for i=1,3 do
		self.current[i].icon:SetDesaturated(desaturate)
		for j=1,3 do
			self.current[i].ability[j].icon:SetDesaturated(desaturate)
		end
		self.current[i].health:SetVertexColor(rb,g,rb,1)
	end
	self.lock:SetDesaturated(desaturate)
end

function rematch:KeepButtonOnClick()
	RematchSaved.Settings.KeepFirst = self:GetChecked()
	rematch:UpdatePets()
end

function rematch:AutoButtonOnClick()
	RematchSaved.Settings.AutoShow = self:GetChecked()
end

function rematch:CloseButtonOnClick()
	self:GetParent():Hide()
end

function rematch:UnsaveButtonOnClick()
	RematchSaved[rematch.target]=nil
	rematch:UpdateTarget()
end

function rematch:SavePets()
	RematchSaved[rematch.target] = {}
	for i=1,3 do
		local guid,ability1,ability2,ability3 = C_PetJournal.GetPetLoadOutInfo(i)
		table.insert(RematchSaved[rematch.target],{guid,ability1,ability2,ability3})
	end
	rematch:UpdateTarget()
end

function rematch:SetupPetsToLoad()
	local saved = RematchSaved[rematch.target]
	rematch.petsToLoad = rematch.petsToLoad or {[1]={},[2]={},[3]={}}
	for i=1,3 do
		wipe(rematch.petsToLoad[i])
	end
	if RematchSaved.Settings.KeepFirst then
		local p = rematch.petsToLoad[1] -- avoiding garbage creation
		p[1],p[2],p[3],p[4] = C_PetJournal.GetPetLoadOutInfo(1)
		local currentIdx = 2
		for i=1,3 do -- grab first two saved that aren't the locked pet
			if saved[i][1]~=p[1] and currentIdx<=3 then
				for j=1,4 do
					rematch.petsToLoad[currentIdx][j] = saved[i][j]
				end
				currentIdx = currentIdx + 1
			end
		end
	else
		for i=1,3 do
			for j=1,4 do
				rematch.petsToLoad[i][j] = saved[i][j]
			end
		end
	end
	-- verify all saved pets still exist, remove any missing
  for i=1,3 do
	  if not C_PetJournal.GetPetStats(rematch.petsToLoad[i][1]) then
			wipe(rematch.petsToLoad[i])
		end
	end
end

function rematch:LoadPets()
	local saved = RematchSaved[rematch.target]
	if saved then
		rematch.petsCurrentlyLoaded = rematch.petsCurrentlyLoaded or {}

		rematch.loadButton:Disable()
		rematch.saveButton:Disable()
		rematch.keepButton:Disable()
		rematch.keepButton.text:SetVertexColor(.5,.5,.5,1)
		rematch.unsaveButton:Disable()

		rematch:DesaturatePets(1)

		rematch:SetupPetsToLoad()

		rematch.petsLoading.timer = 1
		rematch.petsLoading:SetScript("OnUpdate",rematch.ProcessLoad)
		rematch.petsLoading:Show()
	end
end

function rematch:PetsNeedLoading()
	if rematch.target and rematch.target~="<no target>" and RematchSaved[rematch.target] then
		rematch:SetupPetsToLoad()
		rematch.petsCurrentlyLoaded = rematch.petsCurrentlyLoaded or {}
		local c = rematch.petsCurrentlyLoaded
		rematch:SetupPetsToLoad()
		for i=1,3 do
			c[1],c[2],c[3],c[4] = C_PetJournal.GetPetLoadOutInfo(i)
			for j=1,4 do
				if c[j]~=rematch.petsToLoad[i][j] then
					return 1
				end
			end
		end
		return nil -- no pets need loaded
	end
end

SlashCmdList["REMATCH"] = function(msg)
	msg=msg:lower()
	if msg=="reset" then
		Rematch:ClearAllPoints()
		Rematch:SetPoint("CENTER",UIParent)
		Rematch:StopMovingFrame()
	elseif msg=="wipeall" then
		RematchSaved = {Settings={}}
		Rematch:StopMovingFrame()
		Rematch:UpdatePets()
		Rematch:UpdateTarget()
		print("Rematch: All saved targets are now wiped.")
	elseif msg=="help" then
		print("Rematch v"..GetAddOnMetadata("Rematch","Version"))
		print("/rematch : toggle the window")
		print("/rematch reset : reset the window position")
		print("/rematch wipeall : delete all saved information")
	else
		Rematch:Toggle()
	end
end
SLASH_REMATCH1 = "/rematch"

--[[ Timer management ]]

function rematch:BuildTimer()
	self.petsChanging = CreateFrame("Frame") -- for outside influences
	self.petsChanging:Hide()
	hooksecurefunc(C_PetJournal,"SetAbility",rematch.StartPetsChanging)
	hooksecurefunc(C_PetJournal,"SetPetLoadOutInfo",rematch.StartPetsChanging)
	self.petsLoading = CreateFrame("Frame") -- for loading pets over time
	self.petsLoading:Hide()
end

function rematch:StartPetsChanging()
	rematch.petsChanging.timer = 0
	rematch.petsChanging:SetScript("OnUpdate",rematch.PetsAreChanging)
	rematch.petsChanging:Show()
end

function rematch:PetsAreChanging(elapsed)
	self.timer = self.timer + elapsed
	if self.timer > 0.1 then
		self:Hide()
		rematch:UpdatePets()
	end
end

function rematch:ProcessLoad(elapsed)
	self.timer = self.timer + elapsed
	if self.timer > 0 then
		self.timer = 0
		local c = rematch.petsCurrentlyLoaded
		for i=1,3 do
			c.guid,c[2],c[3],c[4] = C_PetJournal.GetPetLoadOutInfo(i)
			local petToLoad = rematch.petsToLoad[i][1]
			if petToLoad and c.guid~=petToLoad then -- if pet guid isn't loaded
				C_PetJournal.SetPetLoadOutInfo(i,petToLoad) -- load it
				return
			elseif petToLoad then -- current pet matched guid we wanted
				for j=2,4 do
					if c[j]~=rematch.petsToLoad[i][j] then -- if ability isn't loaded
						C_PetJournal.SetAbility(i,j-1,rematch.petsToLoad[i][j])
						return
					end
				end
			end
		end
		self:Hide() -- if we got this far, pets are all loaded!
		self:SetScript("OnUpdate",nil)
		rematch.loadButton:Enable()
		rematch.saveButton:Enable()
		rematch.keepButton:Enable()
		rematch.keepButton.text:SetVertexColor(1,.82,0,1)
		rematch.unsaveButton:Enable()
		rematch:UpdatePets()

		if PetJournal and PetJournal:IsVisible() then
			PetJournal_UpdatePetLoadOut()
		end

		if RematchSaved.Settings.AutoShow then
			rematch:Hide()
		end

	end
end

--[[ UI construction ]]

function rematch:CreateTextureElement(layer,cx,cy,anchorPoint,relativeTo,relativePoint,xoff,yoff,texture,tx0,tx1,ty0,ty1,orient,anchor2Point,relative2To,relative2Point,xoff2,yoff2)
	local frame = self:CreateTexture(nil,layer)
	if cx and cy then
		frame:SetSize(cx,cy)
	end
	frame:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	if anchor2Point then
		frame:SetPoint(anchor2Point,relative2To,relative2Point,xoff2,yoff2)
	end
	frame:SetTexture(texture)
	if tx0 then
		frame:SetTexCoord(tx0,tx1,ty0,ty1)
	end
	if orient=="horiz" then
		frame:SetHorizTile(true)
	elseif orient=="vert" then
		frame:SetVertTile(true)
	end
	return frame
end

function rematch:CreateSlotButton(cxbg,cxicon,anchorPoint,relativeTo,relativePoint,xoff,yoff,tx0,tx1,ty0,ty1)
	local button = CreateFrame("Frame",nil,self)
	button:EnableMouse(1)
	button:SetSize(cxbg,cxbg)
	button:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	button.bg = button:CreateTexture(nil,"BACKGROUND")
	button.bg:SetAllPoints(true)
	button.bg:SetTexture("Interface\\Buttons\\UI-EmptySlot")
	button.bg:SetTexCoord(0.15625,0.84375,0.15625,0.84375)
	button.icon = button:CreateTexture(nil,"BORDER")
	button.icon:SetSize(cxicon,cxicon)
	button.icon:SetPoint("CENTER")
	button.icon:SetTexCoord(tx0,tx1,ty0,ty1)
	button.highlight = button:CreateTexture(nil,"HIGHLIGHT")
	button.highlight:SetAllPoints(true)
	button.highlight:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	button.highlight:SetTexCoord(13/64,51/64,14/64,52/64) -- off-center texcoords
	button.highlight:SetBlendMode("ADD")
	button.highlight:SetVertexColor(.5,.75,1,.5)
	button:SetScript("OnMouseDown",rematch.StartMovingFrame)
	button:SetScript("OnMouseUp",rematch.StopMovingFrame)
	return button
end

function rematch:CreatePanelButton(buttontype,cx,cy,anchorPoint,relativeTo,relativePoint,xoff,yoff,label,onclick)
	local button = CreateFrame(buttontype,nil,self,buttontype=="Button" and "UIPanelButtonTemplate" or "UICheckButtonTemplate")
	button:SetSize(cx,cy)
	button:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	local fontstring = buttontype=="Button" and button or button.text
	fontstring:SetText(label)
	button:SetScript("OnClick",onclick)
	return button
end

function rematch:BuildUI()
	self:SetSize(270,194)
	self:SetPoint("CENTER")

	self:SetMovable(1)
	self:SetScript("OnMouseDown",rematch.StartMovingFrame)
	self:SetScript("OnMouseUp",rematch.StopMovingFrame)

	-- border+bg
	local topleft = rematch:CreateTextureElement("BORDER",17,17,"TOPLEFT",self,"TOPLEFT",-1,1,"Interface\\PetBattles\\PetJournal",0.03125,0.06445313,0.02246094,0.0390625)
	local topright = rematch:CreateTextureElement("BORDER",17,17,"TOPRIGHT",self,"TOPRIGHT",1,1,"Interface\\PetBattles\\PetJournal",0.06445313,0.03125,0.02246094,0.0390625)
	local bottomleft = rematch:CreateTextureElement("BORDER",17,17,"BOTTOMLEFT",self,"BOTTOMLEFT",-1,-1,"Interface\\PetBattles\\PetJournal",0.03125,0.06445313,0.0390625,0.02246094)
	local bottomright = rematch:CreateTextureElement("BORDER",17,17,"BOTTOMRIGHT",self,"BOTTOMRIGHT",1,-1,"Interface\\PetBattles\\PetJournal",0.06445313,0.03125,0.0390625,0.02246094)
	local top = rematch:CreateTextureElement("BORDER",nil,nil,"TOPLEFT",topleft,"TOPRIGHT",0,0,"Interface\\PetBattles\\_PetJournalHorizTile",0,1,0.09375,0.21875,"horiz","BOTTOMRIGHT",topright,"BOTTOMLEFT",0,1)
	local bottom = rematch:CreateTextureElement("BORDER",nil,nil,"TOPLEFT",bottomleft,"TOPRIGHT",0,-1,"Interface\\PetBattles\\_PetJournalHorizTile",0,1,0.21875,0.09375,"horiz","BOTTOMRIGHT",bottomright,"BOTTOMLEFT")
	local left = rematch:CreateTextureElement("BORDER",nil,nil,"TOPLEFT",topleft,"BOTTOMLEFT",0,0,"Interface\\PetBattles\\!BattleSlotFrame-Left",0,1,0,1,"vert","BOTTOMRIGHT",bottomleft,"TOPRIGHT",-1,0)
	local right = rematch:CreateTextureElement("BORDER",nil,nil,"TOPLEFT",topright,"BOTTOMLEFT",1,0,"Interface\\PetBattles\\!BattleSlotFrame-Left",1,0,0,1,"vert","BOTTOMRIGHT",bottomright,"TOPRIGHT")

	-- current pets header frame
	local headerframe = CreateFrame("Frame","RematchHeader",self)
	-- header text
	local headertext = headerframe:CreateFontString(nil,"OVERLAY","GameFontNormal")
	headertext:SetJustifyH("CENTER")
	headertext:SetText("Current Battle Pets")
	headertext:SetSize(0,28)
	headertext:SetPoint("CENTER",top,"TOP",0,-5)
	self.headertext = headertext
	-- header textures
	local headerbg = rematch.CreateTextureElement(headerframe,"BACKGROUND",0,18,"LEFT",headertext,"LEFT",-10,1,"Interface\\PetBattles\\_PetJournalHorizTile",0,1,0.234375,0.421875,nil,"RIGHT",headertext,"RIGHT",10,0)
	local headerf = rematch.CreateTextureElement(headerframe,"ARTWORK",nil,nil,"TOPLEFT",headertext,"TOPLEFT",0,1,"Interface\\PetBattles\\_PetJournalHorizTile",0,1,0.4375,0.75,"horiz","BOTTOMRIGHT",headertext,"BOTTOMRIGHT")
	local headerleft = rematch.CreateTextureElement(headerframe,"OVERLAY",28,28,"RIGHT",headerf,"LEFT",0,0,"Interface\\PetBattles\\PetJournal",0.234375,0.3125,0.06347656,0.10253906)
	local headerright = rematch.CreateTextureElement(headerframe,"OVERLAY",28,28,"LEFT",headerf,"RIGHT",0,0,"Interface\\PetBattles\\PetJournal",0.234375,0.3125,0.02246094,0.06152344)

	-- current pet slots
	self.current = {}
	for i=1,3 do
		self.current[i] = rematch:CreateSlotButton(48,39,"TOPLEFT",self,"TOPLEFT",(i-1)*82+16,-20,0.075,0.925,0.075,0.925)
		local button = self.current[i]
		button.ability = {}
		for j=1,3 do
			button.ability[j] = rematch.CreateSlotButton(button,22,16,"TOPLEFT",button,"TOPRIGHT",2,(j-1)*-22,0.075,0.925,0.075,0.925)
			button.ability[j]:SetScript("OnEnter",self.DisplayAbilityTooltip)
			button.ability[j]:SetScript("OnLeave",self.HideAbilityTooltip)
		end
		local healthbg = rematch.CreateTextureElement(button,"BORDER",43,12,"TOPLEFT",button,"BOTTOMLEFT",2,-2,"Interface\\ARCHEOLOGY\\Arch-Progress-BG",3/512,429/512,3/64,41/64)
		button.health = rematch.CreateTextureElement(button,"OVERLAY",38,6,"TOPLEFT",healthbg,"TOPLEFT",3,-3,"Interface\\TARGETINGFRAME\\UI-TargetingFrame-BarFill")
		button.health:SetVertexColor(0,.85,0,1)
		button.dead = rematch.CreateTextureElement(button,"ARTWORK",38,38,"CENTER",button,"CENTER",0,0,"Interface\\PetBattles\\DeadPetIcon")
		button:SetScript("OnEnter",self.DisplayPetTooltip)
		button:SetScript("OnLeave",self.HidePetTooltip)
	end

	self.lock = rematch.CreateTextureElement(self.current[1],"OVERLAY",22,22,"BOTTOMRIGHT",self.current[1],"BOTTOMRIGHT",-1,2,"Interface\\PetBattles\\PetBattle-LockIcon")

--	local separator = rematch:CreateTextureElement("ARTWORK",64,9,"LEFT",left,"TOPRIGHT",-11,-88,"Interface\\PetBattles\\_PetJournalHorizTile",	0,1,0.0078125,0.078125,"horiz","RIGHT",right,"TOPLEFT",12,-88)
	local separator = rematch:CreateTextureElement("ARTWORK",64,9,"TOPLEFT",self,"TOPLEFT",3,-95,"Interface\\PetBattles\\_PetJournalHorizTile",	0,1,0.0078125,0.078125,"horiz","TOPRIGHT",self,"TOPRIGHT",-3,-95)

	-- saved pets header frame
	local savedheaderframe = CreateFrame("Frame",nil,self)
	local savedheadertext = headerframe:CreateFontString(nil,"OVERLAY","GameFontNormal")
	savedheadertext:SetJustifyH("CENTER")
	savedheadertext:SetText("Saved for <no target>")
	savedheadertext:SetSize(0,28)
	savedheadertext:SetPoint("CENTER",separator,"TOP",0,-6)
	self.savedheadertext = savedheadertext
	local savedheaderbg = rematch.CreateTextureElement(savedheaderframe,"BACKGROUND",0,18,"LEFT",savedheadertext,"LEFT",-10,1,"Interface\\PetBattles\\_PetJournalHorizTile",0,1,0.234375,0.421875,nil,"RIGHT",savedheadertext,"RIGHT",10,0)
	local savedheaderf = rematch.CreateTextureElement(savedheaderframe,"ARTWORK",nil,nil,"TOPLEFT",savedheadertext,"TOPLEFT",0,1,"Interface\\PetBattles\\_PetJournalHorizTile",0,1,0.4375,0.75,"horiz","BOTTOMRIGHT",savedheadertext,"BOTTOMRIGHT")
	local savedheaderleft = rematch.CreateTextureElement(savedheaderframe,"OVERLAY",28,28,"RIGHT",savedheaderf,"LEFT",0,0,"Interface\\PetBattles\\PetJournal",0.234375,0.3125,0.06347656,0.10253906)
	local savedheaderright = rematch.CreateTextureElement(savedheaderframe,"OVERLAY",28,28,"LEFT",savedheaderf,"RIGHT",0,0,"Interface\\PetBattles\\PetJournal",0.234375,0.3125,0.02246094,0.06152344)

	self.topbg = rematch:CreateTextureElement("BACKGROUND",nil,nil,"TOPLEFT",self,"TOPLEFT",3,-3,"Interface\\PetBattles\\PetJournal",120/512,404/512,499/1024,604/1024,nil,"BOTTOMRIGHT",separator,"BOTTOMRIGHT",0,3)
	rematch:CreateTextureElement("BACKGROUND",nil,nil,"TOPLEFT",separator,"TOPLEFT",0,-3,"Interface\\PetBattles\\PetJournal",120/512,404/512,499/1024,590/1024,nil,"BOTTOMRIGHT",self,"BOTTOMRIGHT",-3,3)

	-- saved pet slots
	self.saved = {}
	for i=1,3 do
		self.saved[i] = rematch:CreateSlotButton(42,34,"TOPLEFT",separator,"TOPLEFT",(i-1)*46+12,-22,0.075,0.925,0.075,0.925)
		self.saved[i]:SetScript("OnEnter",self.DisplayPetTooltip)
		self.saved[i]:SetScript("OnLeave",self.HidePetTooltip)
	end

	-- panel buttons to right of saved slots
	self.saveButton = rematch:CreatePanelButton("Button",64,24,"BOTTOMRIGHT",self,"BOTTOM",-1,8,"Save",rematch.SavePets)
	self.loadButton = rematch:CreatePanelButton("Button",64,24,"BOTTOMLEFT",self,"BOTTOM",1,8,"Load",rematch.LoadPets)
	self.keepButton = rematch:CreatePanelButton("CheckButton",24,24,"TOPLEFT",self.saved[3],"TOPRIGHT",6,2,"Keep first pet",rematch.KeepButtonOnClick)
	self.autoButton = rematch:CreatePanelButton("CheckButton",24,24,"TOPLEFT",self.keepButton,"BOTTOMLEFT",0,3,"Auto Show",rematch.AutoButtonOnClick)

	self.closeButton = rematch:CreatePanelButton("Button",60,24,"TOPLEFT",self.loadButton,"TOPRIGHT",1,0,"Close",rematch.CloseButtonOnClick)
	self.unsaveButton = rematch:CreatePanelButton("Button",60,24,"TOPRIGHT",self.saveButton,"TOPLEFT",-1,0,"Delete",rematch.UnsaveButtonOnClick)

	table.insert(UISpecialFrames,"Rematch")
	self:SetClampedToScreen(1)

	self.petTooltip = CreateFrame("GameTooltip","RematchPetTooltip",self,"GameTooltipTemplate")

	if RematchSaved.Settings.XPos then
		self:ClearAllPoints()
		self:SetPoint("BOTTOMLEFT",UIParent,RematchSaved.Settings.XPos,RematchSaved.Settings.YPos)
	end

end

--[[ Pet/Ability tooltips ]]

local vulnerabilities = {{"Humanoid",4,5},{"Dragonkin",1,3},{"Flying",6,8},{"Undead",5,2},{"Critter",8,7},{"Magic",2,9},{"Elemental",9,10},{"Beast",10,1},{"Aquatic",3,4},{"Mechanical",7,6}}

function rematch:PetIconAsText(petType,size)
	if type(petType)=="number" and petType>0 and petType<11 then
		return "\124TInterface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType]..":"..size..":"..size..":0:0:128:256:102:63:129:168\124t"
	else
		return "\124TInterface\\Icons\\INV_Misc_QuestionMark:"..size.."\124t"
	end
end

-- default's ability tooltip is adequate
function rematch:DisplayAbilityTooltip()
	if self:GetParent().petID then
		local _,maxHealth,power,speed = C_PetJournal.GetPetStats(self:GetParent().petID)
		FloatingPetBattleAbility_Show(self.abilityID,maxHealth,power,speed)
		FloatingPetBattleAbilityTooltip:ClearAllPoints()
		FloatingPetBattleAbilityTooltip:SetPoint("TOPLEFT",self,"TOPRIGHT")
	end
end
function rematch:HideAbilityTooltip()
	FloatingPetBattleAbilityTooltip:Hide()
end

-- not going to load PetBattleUI for a decent pet tooltip, create our own
function rematch:DisplayPetTooltip()
	if self.petID then
		local _,altName,level,xp,maxXp,_,_,realName,icon,petType = C_PetJournal.GetPetInfoByPetID(self.petID)
		local health,maxHealth,power,speed,breedQuality = C_PetJournal.GetPetStats(self.petID)
		if health then
			local tooltip = rematch.petTooltip
			tooltip:SetOwner(self,"ANCHOR_NONE")
			tooltip:SetPoint("TOPLEFT",self.ability and self.ability[1] or self,"TOPRIGHT")
			tooltip:AddLine(rematch:PetIconAsText(petType,30).." "..(altName or realName),1,1,1)
			if altName then
				tooltip:AddLine(realName,1,.82,0)
			end
			tooltip:AddDoubleLine("\124TInterface\\PetBattles\\PetBattle-StatIcons:12:12:0:0:32:32:16:32:16:32\124t "..health.."/"..maxHealth,"Level "..level,1,1,1,1,1,1)
			tooltip:AddDoubleLine("\124TInterface\\PetBattles\\PetBattle-StatIcons:12:12:0:0:32:32:0:15:0:15\124t "..power,vulnerabilities[petType][1],1,1,1,1,1,1)
			tooltip:AddLine("\124TInterface\\PetBattles\\PetBattle-StatIcons:12:12:0:0:32:32:0:15:16:32\124t "..speed,1,1,1)
			tooltip:AddLine(" ")
			tooltip:AddLine("Damage taken",1,.82,0)
			tooltip:AddLine("\124TInterface\\PetBattles\\BattleBar-AbilityBadge-Strong:32\124tfrom "..rematch:PetIconAsText(vulnerabilities[petType][2],28).." abilities",1,1,1)
			tooltip:AddLine("\124TInterface\\PetBattles\\BattleBar-AbilityBadge-Weak:32\124tfrom "..rematch:PetIconAsText(vulnerabilities[petType][3],28).." abilities",1,1,1)
			tooltip:SetBackdropColor(0,0,0,1)
			tooltip:Show()
		end
	end
end
function rematch:HidePetTooltip()
	rematch.petTooltip:Hide()
end
